//****************************************************************************
//**
//**    mVec.cpp
//**    Implementation for various mVec* functions.
//**    (C) 2000 - Douglas H. Cox
//**
//****************************************************************************
//============================================================================
//    HEADERS
//============================================================================
#include "mVec3.h"
#include "mFrame.h"
#include "mOCS.h"

//============================================================================
//    PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
//============================================================================

//============================================================================
//    PRIVATE STRUCTURES / UTILITY CLASSES
//============================================================================

//============================================================================
//    PRIVATE DATA
//============================================================================

//============================================================================
//    PRIVATE FUNCTIONS
//============================================================================

//============================================================================
//    FUNCTIONS
//============================================================================

//============================================================================
//    CLASS METHODS
//============================================================================

//=============================================================
// mVec3::setHP
void mVec3::setHP(float heading, float pitch)
{
  mSinCos(heading, v[0], v[1]);
  v[0] = -v[0];
  v[2] = mSin(pitch);   
  normalize();
}

//=============================================================
// mVec3::getHP
void mVec3::getHP(float& heading, float& pitch) const
{
  heading = mATan2(-v[0], v[1]);  
  pitch = mASin(v[2] / mSqrt(v[0]*v[0] + v[1]*v[1]));  
}

// ============================================================
// mVec3::makeNormal
void mVec3::makeNormal(const mVec3& a, const mVec3& b, const mVec3& c)
{
  cross(b - a, c - b);
  normalize();
}

//=============================================================
// mVec3::xformTo
void mVec3::xformTo(const mVec3& vec, const mFrame& f)
{
  mVec3 nv(vec.dot(f.a[0]),
           vec.dot(f.a[1]),
           vec.dot(f.a[2]));
  set(nv);
}

//=============================================================
// mVec3::xformFrom
void mVec3::xformFrom(const mVec3& vec, const mFrame& f)
{
  mVec3 nv(vec.v[0] * f.a[0].v[0] + 
           vec.v[1] * f.a[1].v[0] + 
           vec.v[2] * f.a[2].v[0],
           vec.v[0] * f.a[0].v[1] + 
           vec.v[1] * f.a[1].v[1] + 
           vec.v[2] * f.a[2].v[1],
           vec.v[0] * f.a[0].v[2] + 
           vec.v[1] * f.a[1].v[2] + 
           vec.v[2] * f.a[2].v[2]);
           
  set(nv);
}

//=============================================================
// mVec3::xformTo
void mVec3::xformTo(const mVec3& vec, const mOCS& ocs)
{
  assert(&vec != &ocs.o);
  set(vec);

  if (ocs.type & mOCS::ORIGIN)
    sub(ocs.o);

  if (ocs.type & mOCS::FRAME)
    xformTo(*this, ocs.f);

  if (ocs.type & mOCS::SCALE)
    compDiv(ocs.s);
}

//=============================================================
// mVec3::xformVecTo
void mVec3::xformVecTo(const mVec3& vec, const mOCS& ocs)
{
  assert(&vec != &ocs.o);
  set(vec);

  if (ocs.type & mOCS::FRAME)
    xformTo(*this, ocs.f);

  if (ocs.type & mOCS::SCALE)
    compDiv(ocs.s);
}

//=============================================================
// mVec3::xformFrom
void mVec3::xformFrom(const mVec3& vec, const mOCS& ocs)
{
  assert(&vec != &ocs.o);
  set(vec);

  if (ocs.type & mOCS::SCALE)
    compMult(ocs.s);
  
  if (ocs.type & mOCS::FRAME)
    xformFrom(*this, ocs.f);
  
  if (ocs.type & mOCS::ORIGIN)
    add(ocs.o);
}

//=============================================================
// mVec3::xformVecFrom
void mVec3::xformVecFrom(const mVec3& vec, const mOCS& ocs)
{
  assert(&vec != &ocs.o);
  set(vec);

  if (ocs.type & mOCS::SCALE)
    compMult(ocs.s);
  
  if (ocs.type & mOCS::FRAME)
    xformFrom(*this, ocs.f);
}

//****************************************************************************
//**    END MODULE mVec.cpp
//****************************************************************************

